function[] = fcc_plane_wave(Z,a,n)
%FCC_PLANE_WAVE Bandstructure of fcc metals.
%   fcc_plane_wave(Z,a) generates the bandstructure of an fcc metal with
%   atomic number Z and lattice constant a.
%
%   fcc_plane_wave(Z,a,n) uses the n next nearest neighbors (1 = only
%   nearest, 2 = nearest and next nearest etc.). The default value is 2.
%
%   Z and a should be in SI-Units. n has to be between 1 and 1136.

if nargin < 3, dist_neighbors=2; end;
if nargin == 3, dist_neighbors = n; end

%% This part calculates the position of the neighbors

t=0;

gmin = -20;
gmax = 20;
anz = zeros(3,(gmax-gmin+1)^3);

for i = gmin:gmax
    for j = gmin:gmax
        for l = gmin:gmax
            t=t+1;
            anz(1,t) = (i+j-l);
            anz(2,t) = (-i+j+l);
            anz(3,t) = (+i-j+l);
        end
    end
end

lange = zeros(1,length(anz));

for k = 1:length(anz)
    lange(k) = sqrt(anz(1,k)^2+anz(2,k)^2+anz(3,k)^2);
end

sortiert = sort(lange);
t=1;
gezaehlt = zeros(2,length(sortiert));
gezaehlt(1,1) = sortiert(1);
gezaehlt(2,1) = 1;

for k = 2:length(sortiert);
    
    if sortiert(k) == sortiert(k-1)
        gezaehlt(2,t) = gezaehlt(2,t)+1;
    else
        t = t+1;
        gezaehlt(1,t) = sortiert(k);
        gezaehlt(2,t) = 1;
    end
    
end

distance_matrix = gezaehlt(1:2,1:t);

anz2 = zeros(4,length(anz));
anz2(1:3,:)=anz;
anz2(4,:)=lange;

%%

h_bar = 1.054571726e-34;
mass = 9.10938291e-31; % electron mass
e = -1.602176e-19; % electron charge
epsilon = 8.854187e-12; % vacuum permittivity

lambda = h_bar.^2./(2.*mass);
volume = a^3/4; % volume of the unit cell of fcc.

punkte = 1000; % how many points are calculated between two symmetry points

l_point = [0.5,0.5,0.5].*(2.*pi./a); % symmetry points that are used
g_point = [0,0,0];
x_point = [0,1,0].*(2.*pi./a);
u_point = [0.25,1,0.25].*(2.*pi./a);
w_point = [0.5,1,0].*(2.*pi./a);
k_point = [0.75,0.75,0].*(2.*pi./a);

lg = g_point-l_point; % vectors from point to point
gx = x_point-g_point;
xu = u_point-x_point;
uw = w_point-u_point;
wk = k_point-w_point;
kg = g_point-k_point;

kg_length = sqrt(sum(kg.^2)); % length of the longest vector

lg_length = sqrt(sum(lg.^2))./kg_length; % normed vector length
gx_length = sqrt(sum(gx.^2))./kg_length;
xu_length = sqrt(sum(xu.^2))./kg_length;
uw_length = sqrt(sum(uw.^2))./kg_length;
wk_length = sqrt(sum(wk.^2))./kg_length;

intervall6 = 1:punkte; % construction of x-axis for the plot

intervall1 = (0:punkte).*lg_length;
intervall2 = (1:punkte).*gx_length + intervall1(end);
intervall3 = (1:punkte).*xu_length + intervall2(end);
intervall4 = (1:punkte).*uw_length + intervall3(end);
intervall5 = (1:punkte).*wk_length + intervall4(end);
intervall6 = intervall6  + intervall5(end);

xachse = [intervall1,intervall2,intervall3,intervall4,intervall5,intervall6];

lg_l = zeros(3,punkte+1); % all k values between the symmetry points where eigenvalues are calculated
lg_l(1,:) = linspace(l_point(1),g_point(1),punkte+1);
lg_l(2,:) = linspace(l_point(2),g_point(2),punkte+1);
lg_l(3,:) = linspace(l_point(3),g_point(3),punkte+1);

gx_l = zeros(3,punkte+1);
gx_l(1,:) = linspace(g_point(1),x_point(1),punkte+1);
gx_l(2,:) = linspace(g_point(2),x_point(2),punkte+1);
gx_l(3,:) = linspace(g_point(3),x_point(3),punkte+1);

xu_l = zeros(3,punkte+1);
xu_l(1,:) = linspace(x_point(1),u_point(1),punkte+1);
xu_l(2,:) = linspace(x_point(2),u_point(2),punkte+1);
xu_l(3,:) = linspace(x_point(3),u_point(3),punkte+1);

uw_l = zeros(3,punkte+1);
uw_l(1,:) = linspace(u_point(1),w_point(1),punkte+1);
uw_l(2,:) = linspace(u_point(2),w_point(2),punkte+1);
uw_l(3,:) = linspace(u_point(3),w_point(3),punkte+1);

wk_l = zeros(3,punkte+1);
wk_l(1,:) = linspace(w_point(1),k_point(1),punkte+1);
wk_l(2,:) = linspace(w_point(2),k_point(2),punkte+1);
wk_l(3,:) = linspace(w_point(3),k_point(3),punkte+1);

kg_l = zeros(3,punkte+1);
kg_l(1,:) = linspace(k_point(1),g_point(1),punkte+1);
kg_l(2,:) = linspace(k_point(2),g_point(2),punkte+1);
kg_l(3,:) = linspace(k_point(3),g_point(3),punkte+1);


combi = [lg_l,gx_l(:,2:end),xu_l(:,2:end),uw_l(:,2:end),wk_l(:,2:end),kg_l(:,2:end)];

% Eigenwerte berechnen

nr_of_neighbors = sum(distance_matrix(2,1:dist_neighbors+1));
neighbors = zeros(3,nr_of_neighbors);

t = 0;

for durch = 1:length(anz2)
    if anz2(4,durch) <= distance_matrix(1,dist_neighbors+1)
        t = t+1;
        neighbors(1:3,t) = anz2(1:3,durch);
    end
end

M2 = zeros(nr_of_neighbors); % Matrix

neighbors_recip = neighbors.*2.*pi./a; % neighbors is in units of 2*pi/a

for i = 1:nr_of_neighbors
    for j = 1:nr_of_neighbors
        if i ~= j
            M2(i,j) = -Z * e^2 / volume / epsilon / ((neighbors_recip(:,i)-neighbors_recip(:,j))'*(neighbors_recip(:,i)-neighbors_recip(:,j)));
        end
    end
end

energy = zeros(nr_of_neighbors,size(combi,2)); % eigenvalues of all k values

for run3 = 1:size(combi,2) % loop over all k values
    
    k = combi(:,run3); % get k
   
    for run1 = 1:nr_of_neighbors
         M2(run1,run1) = lambda*((k-neighbors_recip(:,run1))'*(k-neighbors_recip(:,run1)));
    end
    
    energy(:,run3) = eig(M2); % get eigenvalues
        
end

% plots

figure('Name','Plot of Planewave-Method for FCC-Crystal')
for run4 = 1:nr_of_neighbors
plot(xachse,energy(run4,:))
hold on
end

title(['Z = ',num2str(Z),'; a = ',num2str(a*1e10),' ',char(197),'; Neighbors: ',num2str(nr_of_neighbors)])
xlim([0 xachse(end)])
my_xticklabels([0 intervall1(end) intervall2(end) intervall3(end) intervall4(end) intervall5(end) intervall6(end)],{'L','\Gamma','X','U','W','K','\Gamma'});
set(gca,'XGrid','on');