function fcc_plane_wave_gui()
%FCC_PLANE_WAVE_GUI     Graphical user interface for the calculation of the
%                       bandstructure from fcc crystals using the plane wave method.
%
%   fcc_plane_wave_gui() generates a graphical user interface where you
%   are able to choose from predefined fcc crystals or set your own atomic
%   number Z and lattice constant a. As well you have to set the number of
%   nearest neighbours n for the calculation. 
%   After pressing the calculation button a new figure, that plots the 
%   bandstructure of your chosen atom at 1 atm and room temperature 
%   (in case of inert gases at 4 K), will be generated.
%   
%   Step by step:
%
%   1. choose a predefined atom e.g. 'Copper' or for self-defined values
%   'Other'
%
%   2. check/set the atomic number Z and lattice constant a (a in angstrom)
%
%   3. set the number of nearest neighbours n e.g. '2' for second nearest 
%   neighbours (high numbers leads to high usage of memory)
%
%   4. press the calculation button
%
%   5. if the input was correct and no error message appears you will see
%   the bandstructure of your chosen atom
%
%   For more information about the fcc_plane_wave function open its help.
%   
%   See also:
%   fcc_plane_wave

%% This part creates the graphical input like the editboxes, labels, dropdownbox etc.
fig1=figure('Name','Planewave-Method for FCC-Crystals','Resize','off','Menubar','none');
fig1_pos=get(fig1,'Position');
fig1_pos=[fig1_pos(1:2),200,200];
set(fig1,'Position',fig1_pos)
hedit_Z=uicontrol('Style','edit',...
    'Position',[fig1_pos(3)*2/14,fig1_pos(4)*3/6,40 20],'Enable','inactive');
hedit_a=uicontrol('Style','edit',...
    'Position',[fig1_pos(3)*9/14,fig1_pos(4)*3/6,40 20],'Enable','inactive');
edit_Z_pos=get(hedit_Z,'Position');
edit_a_pos=get(hedit_a,'Position');
hedit_n=uicontrol('Style','edit',...
    'Position',[fig1_pos(3)/3+10 fig1_pos(4)*3/12-15 40 20]);
edit_n_pos=get(hedit_n,'Position');
pop1=uicontrol('Style','popup',...
    'String','Ne:Neon (4K)|Al:Aluminum|Ar:Argon (4K)|Ca:Calcium|Ni:Nickel|Cu:Copper|Kr:Krypton (4K)|Sr:Strontium|Rh:Rhodium|Pd:Palladium|Ag:Silver|Xe:Xenon (4K)|Ce:Cerium|Yb:Ytterbium|Ir:Iridium|Pt:Platinum|Au:Gold|Pb:Lead|Ac:Actinium|Th:Thorium|Other',...
    'Position',[fig1_pos(3)/4-5, fig1_pos(4)*2/3, 105, 20],...
    'Callback',{@setatom,hedit_Z,hedit_a});
pop1_pos=get(pop1,'Position');
lb1=uicontrol('Style','text',...
    'String','Choose an atom from the list:',...
    'Position',[pop1_pos(1)-21, pop1_pos(2)+25, 150, 15]);
lb2=uicontrol('Style','text',...
    'String','Z:');
set(lb2,'Position',[edit_Z_pos(1)-16 edit_Z_pos(2)+2 13 16]);
lb3=uicontrol('Style','text',...
    'String','a:');
set(lb3,'Position',[edit_a_pos(1)-16 edit_a_pos(2)+2 13 16]);
lb4=uicontrol('Style','text',...
    'String',char(197));
set(lb4,'Position',[edit_a_pos(1)+43 edit_a_pos(2)+2 13 16]);
lb5=uicontrol('Style','text',...
    'String','Choose number of nearest neighbours (attn. on memory):');
set(lb5,'Position',[edit_n_pos(1)-60 edit_n_pos(2)+25 153 30]);
lb6=uicontrol('Style','text',...
    'String','n:');
set(lb6,'Position',[edit_n_pos(1)-16 edit_n_pos(2)+2 13 16]);

pB_eval=uicontrol('Style','pushbutton','String','Calculate',...
    'Position',[fig1_pos(3)/3 fig1_pos(4)/24 60 20],...
    'Callback',{@fcc_calc,hedit_Z,hedit_a,hedit_n});

end

%% This part is for the callback function of the dropdownbox that sets the chosen atom
function [Z,a]=setatom(hObj,event,edit_Z,edit_a)
contents=cellstr(get(hObj,'String'));
atom=contents(get(hObj,'Value'));
atom=atom{1};

set(edit_Z,'Enable','off');
set(edit_a,'Enable','off');

switch atom(1:2)
    case 'Cu'
        Z = 29;
        a = 3.61e-10;
    case 'Ag'
        Z = 47;
        a = 4.09e-10;
    case 'Au'
        Z = 79;
        a = 4.08e-10;
    case 'Al'
        Z = 13;
        a = 4.05e-10;
    case 'Ca'
        Z = 20;
        a = 5.58e-10;
    case 'Pb'
        Z = 82;
        a = 4.95e-10;
    case 'Ne'
        Z = 10;
        a = 4.46e-10;
    case 'Ar'
        Z = 18;
        a = 5.31e-10;
    case 'Kr'
        Z = 36;
        a = 5.64e-10;
    case 'Xe'
        Z = 54;
        a = 6.13e-10;
    case 'Ni'
        Z = 28;
        a = 3.52e-10;
    case 'Sr'
        Z = 38;
        a = 6.08e-10;
    case 'Rh'
        Z = 45;
        a = 3.80e-10;
    case 'Pd'
        Z = 46;
        a = 3.89e-10;
    case 'Ir'
        Z = 77;
        a = 3.84-10;
    case 'Pt'
        Z = 78;
        a = 3.92e-10;
    case 'Ac'
        Z = 89;
        a = 5.31e-10;
    case 'Ce'
        Z = 58;
        a = 5.16e-10;
    case 'Yb'
        Z = 70;
        a = 5.48e-10;
    case 'Th'
        Z = 90;
        a = 5.08e-10;
    case 'Ot'
        set(edit_Z,'Enable','on');
        set(edit_a,'Enable','on');
        Z=[];
        a=Z;
    otherwise
        error('Problem by choosing an input! Please try again!')
        
end

set(edit_Z,'String',num2str(Z));
set(edit_a,'String',num2str(a*1e+10));
end

%% This part is for the callback function of the calculationbutton that checks the inputs for the fcc_plane_wave(Z,a,n) function
function fcc_calc(hObj,event,edit_Z,edit_a,edit_n)
    Z=str2num(get(edit_Z,'String'));
    a=str2num(get(edit_a,'String'))*1e-10;
    n=str2num(get(edit_n,'String'));

    if isempty(Z), error('Z is missing!'), end
    if isempty(a), error('a is missing!'), end
    if isempty(n), error('n is missing!'), end
    
    fcc_plane_wave(Z,a,n)
    
end